.big-column {
  width: 25%;
}

.bottom-centered-content {
  margin-top: 5px;
  margin-left: 5%;
  margin-right: auto;
  width: 90%;
  display: block;
}

.centered-text {
  text-align: center;
}

.right-text {
  text-align: right;
}

hr {
  background: #000000;
  height: 1px;
}

label {
  margin-bottom: 4px;
}

label:after {
  content: "" !important;
}

.left-aligned-text {
  text-align: left;
}

.normal-font-weight {
  font-weight: normal;
}

.medium-column {
  width: 15%;
}

.middle-vertical-aligned {
  vertical-align: middle;
}

.small-column {
  width: 5%;
}

.spaced-left {
  margin-left: 10px;
}

.top-centered-content {
  margin-left: 5%;
  margin-right: auto;
  width: 70%;
}

td, th {
  text-align: center;
  vertical-align: middle !important;
}

.wrapper {
  min-height: 500px;
}
.tab {
  width: 100%;
}

.tab-menu {
  width: calc(100%/2);
  height: 60px;
  line-height: 60px;
  font-size: 16px;
  text-align: center;
  color: #000;
  border-bottom: 3px solid #1f71b5;
  background-color: #d9d9d9;
  display: block;
  float: left;
  text-align: center;
  font-weight: bold;
  transition: all 0.2s ease;
}
.tab-menu:hover {
  opacity: 0.75;
}
input[name="tab-menu"] {
  display: none;
}
.tab-contents {
  display: none;
  clear: both;
  overflow: hidden;
  transition: .5s opacity;
  box-shadow: 0 0 3px rgba(0,0,0,.2);
}
#menu1:checked ~ #menu1,
#menu2:checked ~ #menu2 {
  display: block;
}
.tab input:checked + .tab-menu {
  background-color: #1f71b5;
  color: #fff;
}
