.centered-content {
  margin-left: 5%;
  margin-right: auto;
  width: 90%
}

.divider {
  background: #000000;
  height: 1px;
}

.right-aligned {
  float: right;
}

.spaced-left {
  margin-left: 10px;
}

.unapproved {
  color: #808080;
  background-color: #EEEEEE;
}

.wrap-text {
  word-wrap: break-word;
}

.multiple-line {
  white-space: pre-wrap;
  word-wrap: break-word;
}

#issue-resolution-table {
  height: 400px;
  overflow-y: scroll;
}

#issue-resolution {
  width: 28px !important;
  height: 28px !important;
}

.char-count {
  text-align: right;
  color: #808080;
}

.comment-count {
  color: #808080;
}
