.bubble{
    position: relative;
    display:  inline-block;
    margin-right: 5px;
}
.bubble img{
    width: 10px;
}
.bubble #issue-resolution{
    width: 15px;
    height: 15px
}
.description {
    display: none;
    position: absolute;
    bottom: 100%;  
    padding: 5px;
    font-size: 9px;
    text-align: center;
    color: #fff;
    border-radius: 5px;
    background: #000;
    width: 125px;
}
.description:before {
    content: "";
    position: absolute;
    top: 100%;
    left: 50%;
    border: 5px solid transparent;
    border-top: 5px solid #000;
    margin-left: -15px;
}
.bubble:hover .description{
    display: inline-block;
    top: -25px;
    left: -40px;
}