.big-column {
  width: 24%;
}

.bottom-centered-content {
  margin-left: 5%;
  margin-right: auto;
  width: 90%
}

.centered-text {
  text-align: center;
}

hr {
  background: #000000;
  height: 1px;
}

label {
  margin-bottom: 4px;
}

label:after {
  content: "" !important;
}

.left-aligned-text {
  text-align: left;
}

.normal-font-weight {
  font-weight: normal;
}

.medium-column {
  width: 16%;
}

.middle-vertical-aligned {
  vertical-align: middle;
}

.small-column {
  width: 12%;
}

.spaced-left {
  margin-left: 10px;
}

.top-centered-content {
  margin-left: 5%;
  margin-right: auto;
  width: 70%;
}

td, th {
  text-align: center;
  vertical-align: middle !important;
}

.wrapper {
  min-height: 500px;
}

.division-flex-outer {
  display: flex;
  align-items: center;
  justify-content: center;
}

.division-flex-inner-icon {
  width: 30px;
}

.division-flex-inner-num {
  width: 18px;
  text-align: right;
  margin-left: 1px;
}

#issue-resolution {
  width: 28px !important;
  height: 28px !important;
}